"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiteralCompiler = void 0;
var utils_1 = require("../utils");
var compile_1 = require("./compile");
/**
 * literal parse, include string, integer, array, map, bool data structure
 * @require {method} getReferences
 */
var LiteralCompiler = /** @class */ (function (_super) {
    __extends(LiteralCompiler, _super);
    function LiteralCompiler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * 字面量求值，主要包括string, integer, array, map四种数据结构
     * @param literal {object} 定义于velocity.yy文件，type描述数据类型，value属性
     * 是literal值描述
     * @return js variable
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    LiteralCompiler.prototype.getLiteral = function (literal) {
        var _this = this;
        var type = literal.type;
        if (type === 'string') {
            return this.getString(literal);
        }
        if (type === 'integer') {
            return parseInt(literal.value, 10);
        }
        if (type === 'decimal') {
            return parseFloat(literal.value);
        }
        if (type === 'array') {
            return this.getArray(literal);
        }
        if (type === 'map') {
            var ret_1 = {};
            var map_1 = literal.value || {};
            Object.keys(map_1).forEach(function (key) {
                ret_1[key] = _this.getLiteral(map_1[key]);
            });
            return ret_1;
        }
        if (type === 'bool') {
            return {
                true: true,
                false: false,
                null: null,
            }[literal.value];
        }
        return this.getReferences(literal);
    };
    /**
     * 对字符串求值，对已双引号字符串，需要做变量替换
     */
    LiteralCompiler.prototype.getString = function (literal) {
        var val = literal.value;
        if (literal.isEval && (val.indexOf('#') !== -1 || val.indexOf('$') !== -1)) {
            return this.evalStr(val);
        }
        return val;
    };
    /**
     * 对array字面量求值，比如[1, 2]=> [1,2]，[1..5] => [1,2,3,4,5]
     * @param literal {object} array字面量的描述对象，分为普通数组和range数组两种
     * ，和js基本一致
     * @return {array} 求值得到的数组
     */
    LiteralCompiler.prototype.getArray = function (literal) {
        var _this = this;
        if (!('isRange' in literal && literal.isRange === true)) {
            return literal.value.map(function (exp) { return _this.getLiteral(exp); });
        }
        var _a = literal.value.map(function (exp) {
            if (typeof exp === 'string') {
                return parseInt(exp, 10);
            }
            if (typeof exp === 'number') {
                return exp;
            }
            return _this.getReferences(exp);
        }), begin = _a[0], end = _a[1];
        if (Number.isNaN(begin) || Number.isNaN(end)) {
            return [];
        }
        var i = 0;
        var ret = [];
        if (begin < end) {
            for (i = begin; i <= end; i++)
                ret.push(i);
        }
        else {
            for (i = begin; i >= end; i--)
                ret.push(i);
        }
        return ret;
    };
    return LiteralCompiler;
}(compile_1.Compile));
exports.LiteralCompiler = LiteralCompiler;
(0, utils_1.applyMixins)(compile_1.Compile, [LiteralCompiler]);
//# sourceMappingURL=literal.js.map