"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRefText = void 0;
/**
 * 获取引用文本，当引用自身不存在的情况下，需要返回原来的模板字符串
 * get variable text
 */
function getRefText(ast) {
    var _a;
    if (ast.type === 'macro_call') {
        return "#".concat(getMethodText(ast));
    }
    var ret = (ast === null || ast === void 0 ? void 0 : ast.leader) || '';
    var isFn = ast.args !== undefined;
    if (ast.isWraped)
        ret += '{';
    if (isFn) {
        ret += getMethodText(ast);
    }
    else {
        ret += ast.id;
    }
    (_a = ast.path) === null || _a === void 0 ? void 0 : _a.forEach(function (ref) {
        if (ref.type === 'method') {
            ret += '.' + getMethodText(ref);
            return;
        }
        if (ref.type === 'index') {
            var text = '';
            var id = ref.id;
            if (id.type === 'integer') {
                text = '' + id.value;
            }
            else if (id.type === 'string') {
                var sign = id.isEval ? '"' : "'";
                text = sign + id.value + sign;
            }
            else {
                text = getRefText(id);
            }
            ret += '[' + text + ']';
        }
        if (ref.type === 'property') {
            ret += '.' + ref.id;
        }
    });
    if (ast.isWraped)
        ret += '}';
    return ret;
}
exports.getRefText = getRefText;
function getMethodText(ref) {
    var args = (ref.args || []).map(function (arg) { return getLiteral(arg); });
    return ref.id + '(' + args.join(',') + ')';
}
function getLiteral(ast) {
    var ret = '';
    switch (ast.type) {
        case 'string': {
            var sign = ast.isEval ? '"' : "'";
            ret = sign + ast.value + sign;
            break;
        }
        case 'integer':
        case 'runt':
        case 'bool': {
            ret = ast.value + '';
            break;
        }
        case 'array': {
            ret = '[';
            ret += ast.value.map(function (arg) { return getLiteral(arg); }).join(', ');
            ret += ']';
            break;
        }
        // map handle?
        default:
            ret = getRefText(ast);
    }
    return ret;
}
//# sourceMappingURL=index.js.map