var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { Compile } from './compile';
import { parse } from '../parse';
import { getRefText } from '../helper';
import { applyMixins, guid } from '../utils';
/**
 * blocks such as if, foreach, macro syntax handler
 */
var BlockCommand = /** @class */ (function (_super) {
    __extends(BlockCommand, _super);
    function BlockCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BlockCommand.prototype.getBlock = function (block) {
        var ast = block[0];
        var ret = '';
        switch (ast.type) {
            case 'if':
                ret = this.getBlockIf(block);
                break;
            case 'foreach':
                ret = this.getBlockEach(block);
                break;
            case 'macro':
                this.setBlockMacro(block);
                break;
            case 'noescape':
                ret = this.renderAstList(block.slice(1));
                break;
            case 'define':
                this.setBlockDefine(block);
                break;
            case 'macro_body':
                ret = this.getMacroBody(block);
                break;
            default:
                ret = this.renderAstList(block);
        }
        return ret || '';
    };
    /**
     * define
     */
    BlockCommand.prototype.setBlockDefine = function (block) {
        var ast = block[0];
        this.defines[ast.id] = block.slice(1);
    };
    /**
     * define macro
     */
    BlockCommand.prototype.setBlockMacro = function (block) {
        var ast = block[0];
        this.macrosStore[ast.id] = {
            asts: block.slice(1),
            args: ast.args,
        };
    };
    BlockCommand.prototype.getMacroBody = function (asts) {
        var ast = asts[0];
        var _block = asts.slice(1);
        var bodyContent = this.eval(_block, {});
        return this.getMacro(ast, bodyContent);
    };
    /**
     * parse macro call
     */
    BlockCommand.prototype.getMacro = function (ast, bodyContent) {
        var _this = this;
        var macro = this.macrosStore[ast.id];
        if (!macro) {
            var fn = this.macros[ast.id];
            if (!fn || typeof fn !== 'function') {
                return '';
            }
            var jsArgs = (ast.args || []).map(function (arg) { return _this.getLiteral(arg); });
            try {
                return fn.apply(this.macros, jsArgs);
            }
            catch (e) {
                var pos = ast.pos;
                var text = getRefText(ast);
                var err = "\n      at ".concat(text, " L/N ").concat(pos.first_line, ":").concat(pos.first_column);
                e.name = '';
                // eslint-disable-next-line @typescript-eslint/restrict-plus-operands
                e.message += err;
                throw e;
            }
        }
        var args = macro.args, asts = macro.asts;
        var callArgs = ast.args;
        var local = { bodyContent: bodyContent };
        var contextId = "macro:".concat(ast.id, ":").concat(guid());
        (args || []).forEach(function (ref, i) {
            if (!('id' in ref)) {
                return;
            }
            local[ref.id] = callArgs[i] ? _this.getLiteral(callArgs[i]) : undefined;
        });
        return this.eval(asts, local, contextId);
    };
    /**
     * eval
     * @param str {array|string} input string
     * @param local {object} local variable
     * @param contextId {=string} optional contextId, this contextId use to find local variable
     * @return {string}
     */
    BlockCommand.prototype.eval = function (str, local, contextId) {
        if (!local) {
            if (Array.isArray(str)) {
                return this.renderAstList(str);
            }
            return this.evalStr(str);
        }
        var asts = Array.isArray(str) ? str : parse(str);
        contextId = contextId !== null && contextId !== void 0 ? contextId : "eval:".concat(guid());
        if (!asts.length) {
            return '';
        }
        this.local[contextId] = local;
        var ret = this.renderAstList(asts, contextId);
        this.local[contextId] = {};
        this.conditions.shift();
        this.contextId = this.conditions[0] || '';
        return ret;
    };
    /**
     * 对双引号字符串进行eval求值，替换其中的变量，只支持最基本的变量类型替换
     */
    BlockCommand.prototype.evalStr = function (str) {
        var asts = parse(str);
        return this.renderAstList(asts, this.contextId);
    };
    /**
     * parse #foreach
     */
    BlockCommand.prototype.getBlockEach = function (block) {
        var _this = this;
        var ast = block[0];
        var _from = this.getLiteral(__assign(__assign({}, ast.from), { pos: ast.pos }));
        var _block = block.slice(1);
        var _to = ast.to;
        var local = {
            foreach: {
                count: 0,
                index: 0,
                hasNext: false,
            },
            velocityCount: 0,
        };
        var ret = '';
        var uid = guid();
        var contextId = "foreach:".concat(uid);
        var type = {}.toString.call(_from);
        if (!_from || (type !== '[object Array]' && type !== '[object Object]')) {
            return '';
        }
        var items = Array.isArray(_from) ? _from : Object.values(_from);
        var len = items.length;
        items.forEach(function (val, i) {
            if (_this.runState.break) {
                return;
            }
            local[_to] = val;
            local.foreach = {
                count: i + 1,
                index: i,
                hasNext: i + 1 < len,
            };
            local.velocityCount = i + 1;
            _this.local[contextId] = local;
            ret += _this.renderAstList(_block, contextId);
        });
        // if foreach items be an empty array, then this code will shift current
        // conditions, but not this._render call, so this will shift parent context
        if (_from === null || _from === void 0 ? void 0 : _from.length) {
            this.runState.break = false;
            // empty current local context object
            this.local[contextId] = {};
            this.conditions.shift();
            this.contextId = this.conditions[0] || '';
        }
        return ret;
    };
    /**
     * parse #if
     */
    BlockCommand.prototype.getBlockIf = function (block) {
        var _this = this;
        var received = false;
        var asts = [];
        block.some(function (ast) {
            var hasCondition = ast.type === 'elseif' || ast.type === 'if';
            var isIF = hasCondition || ast.type === 'else';
            if (!isIF) {
                // if not if/elseif/else, then push to asts, skip when not received
                received && asts.push(ast);
                return;
            }
            // if received, then skip all
            if (received) {
                return true;
            }
            received = hasCondition ? !!_this.getExpression(ast.condition) : true;
        });
        // keep current condition fix #77
        return this.renderAstList(asts, this.contextId);
    };
    return BlockCommand;
}(Compile));
export { BlockCommand };
applyMixins(Compile, [BlockCommand]);
//# sourceMappingURL=blocks.js.map