function hasProperty(context, field) {
    if (typeof context === 'number' || typeof context === 'string') {
        return context[field] || Object.prototype.hasOwnProperty.call(context, field);
    }
    if (!context) {
        return false;
    }
    return field in context;
}
var matchProperty = function (value, notInContext) {
    return function (_a) {
        var property = _a.property, context = _a.context;
        return value === property && (notInContext ? !hasProperty(context, property) : true);
    };
};
var matchStartWith = function (value) {
    return function (_a) {
        var property = _a.property, context = _a.context;
        return property.indexOf(value) === 0 && !(property in context) && property.length > value.length;
    };
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function getter(base, property) {
    // get(1)
    if (typeof property === 'number') {
        return base[property];
    }
    var letter = property.charCodeAt(0);
    var isUpper = letter < 91;
    var ret = base[property];
    if (ret !== undefined) {
        return ret;
    }
    if (isUpper) {
        // Address => address
        property = String.fromCharCode(letter).toLowerCase() + property.slice(1);
    }
    if (!isUpper) {
        // address => Address
        property = String.fromCharCode(letter).toUpperCase() + property.slice(1);
    }
    return base[property];
}
function getSize(obj) {
    if (Array.isArray(obj)) {
        return obj.length;
    }
    if (typeof obj === 'object' && obj !== null) {
        return Object.keys(obj).length;
    }
    return undefined;
}
var handlers = {
    // $foo.get('bar')
    get: {
        match: matchProperty('get', true),
        resolve: function (_a) {
            var context = _a.context, params = _a.params;
            return getter(context, params[0]);
        },
    },
    // $foo.set('a', 'b')
    set: {
        match: matchProperty('set', true),
        resolve: function (_a) {
            var context = _a.context, params = _a.params;
            context[params[0]] = params[1];
            return '';
        },
    },
    // getAddress()
    getValue: {
        match: matchStartWith('get'),
        resolve: function (_a) {
            var context = _a.context, property = _a.property;
            return getter(context, property.slice(3));
        },
    },
    isValue: {
        match: matchStartWith('is'),
        resolve: function (_a) {
            var context = _a.context, property = _a.property;
            return getter(context, property.slice(2));
        },
    },
    // $page.setName(123)
    setValue: {
        match: matchStartWith('set'),
        resolve: function (_a) {
            var context = _a.context, property = _a.property, params = _a.params;
            context[property.slice(3)] = params[0];
            // set value will not output anything
            context.toString = function () {
                return '';
            };
            return context;
        },
    },
    keySet: {
        match: matchProperty('keySet', true),
        resolve: function (_a) {
            var context = _a.context;
            return Object.keys(context);
        },
    },
    entrySet: {
        match: matchProperty('entrySet', true),
        resolve: function (_a) {
            var context = _a.context;
            return Object.keys(context).map(function (key) { return ({
                key: key,
                value: context[key],
            }); });
        },
    },
    size: {
        match: matchProperty('size', true),
        resolve: function (_a) {
            var context = _a.context;
            return getSize(context);
        },
    },
    put: {
        match: matchProperty('put', true),
        resolve: function (_a) {
            var context = _a.context, params = _a.params;
            return (context[params[0]] = params[1]);
        },
    },
    add: {
        match: matchProperty('add', true),
        resolve: function (_a) {
            var context = _a.context, params = _a.params;
            if (typeof context.push !== 'function') {
                return;
            }
            return context.push(params[0]);
        },
    },
    remove: {
        match: matchProperty('remove', true),
        resolve: function (_a) {
            var context = _a.context, params = _a.params;
            if (Array.isArray(context)) {
                var index = params[0];
                if (typeof index !== 'number') {
                    index = context.indexOf(params[0]);
                }
                var ret = context[index];
                context.splice(index, 1);
                return ret;
            }
            if (typeof context === 'object' && context !== null) {
                var ret = context[params[0]];
                delete context[params[0]];
                return ret;
            }
            return undefined;
        },
    },
    subList: {
        match: matchProperty('subList', true),
        resolve: function (_a) {
            var context = _a.context, params = _a.params;
            return context.slice(params[0], params[1]);
        },
    },
};
var keys = Object.keys(handlers);
export default keys.map(function (key) { return ({
    uid: 'system: ' + key,
    match: handlers[key].match,
    resolve: handlers[key].resolve,
}); });
//# sourceMappingURL=methods.js.map