var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import debugBase from 'debug';
import { getRefText } from '../helper';
import { Compile } from './compile';
import { applyMixins, convert } from '../utils';
var debug = debugBase('velocity');
var posUnknown = { first_line: 'unknown', first_column: 'unknown' };
var References = /** @class */ (function (_super) {
    __extends(References, _super);
    function References() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * get variable value
     * @param {object} ast ast data
     * @param {bool} isVal for example `$foo`, isVal value should be true, other condition,
     * `#set($foo = $bar)`, the $bar value get, isVal set to false
     */
    References.prototype.getReferences = function (ast, isVal) {
        var _this = this;
        if (ast.prue) {
            var define = this.defines[ast.id];
            if (Array.isArray(define)) {
                return this.renderAstList(define);
            }
            if (this.config.unescape && ast.id in this.config.unescape) {
                ast.prue = false;
            }
        }
        var escape = this.config.escape;
        var isSilent = this.silence || ast.leader === '$!';
        var isFunction = ast.args !== undefined;
        var context = this.context;
        var ret = context[ast.id];
        var local = this.getLocal(ast);
        var text = getRefText(ast);
        if (text in context) {
            return ast.prue && escape ? convert(context[text]) : context[text];
        }
        if (ret !== undefined && isFunction) {
            ret = this.getPropMethod(ast, context, ast);
        }
        if (local.isLocal)
            ret = local['value'];
        if (Array.isArray(ast.path)) {
            ast.path.some(function (property) {
                if (ret === undefined) {
                    _this._throw(ast, property);
                }
                // 第三个参数，返回后面的参数ast
                ret = _this.getAttributes(property, ret, ast);
            });
        }
        if (isVal && ret === undefined) {
            ret = isSilent ? '' : getRefText(ast);
        }
        ret = ast.prue && escape ? convert(ret) : ret;
        return ret;
    };
    /**
     * 获取局部变量，在macro和foreach循环中使用
     */
    References.prototype.getLocal = function (ast) {
        var id = ast.id;
        var local = this.local;
        var ret = false;
        var isLocal = this.conditions.some(function (contextId) {
            var hasData = id in local[contextId];
            if (hasData) {
                ret = local[contextId][id];
            }
            return hasData;
        });
        return {
            value: ret,
            isLocal: isLocal,
        };
    };
    /**
     * $foo.bar 属性求值，最后面两个参数在用户传递的函数中用到
     * @param {object} property 属性描述，一个对象，主要包括id，type等定义
     * @param {object} baseRef 当前执行链结果，比如$a.b.c，第一次baseRef是$a,
     * 第二次是$a.b返回值
     * @private
     */
    References.prototype.getAttributes = function (property, baseRef, ast) {
        // fix #54
        if (baseRef === null || baseRef === undefined) {
            return undefined;
        }
        /**
         * type对应着velocity.yy中的attribute，三种类型: method, index, property
         */
        if (property.type === 'method') {
            return this.getPropMethod(property, baseRef, ast);
        }
        if (property.type === 'property') {
            return baseRef[property.id];
        }
        return this.getPropIndex(property, baseRef);
    };
    /**
     * $foo.bar[1] index求值
     * @private
     */
    References.prototype.getPropIndex = function (property, baseRef) {
        var ast = property.id;
        var key = ast.type === 'references' ? this.getReferences(ast) : ast.value;
        return baseRef[key];
    };
    /**
     * $foo.bar()求值
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    References.prototype.getPropMethod = function (property, baseRef, ast) {
        var _this = this;
        var _a;
        var id = property.id;
        var ret = baseRef[id];
        var args = (property.args || []).map(function (exp) { return _this.getLiteral(exp); }) || [];
        var payload = { property: id, params: args, context: baseRef };
        var matched = (_a = this.config.customMethodHandlers) === null || _a === void 0 ? void 0 : _a.find(function (item) { return item && item.match(payload); });
        if (matched) {
            debug('match custom method handler, uid %s', matched.uid);
            // run custom method handler, we can
            // add some native method which Java can do, for example
            // #set($foo = [1, 2]) $foo.size()
            return matched.resolve(payload);
        }
        if (!ret || !ret.call) {
            this._throw(ast, property, 'TypeError');
            return;
        }
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        var that = this;
        if (typeof baseRef === 'object' && baseRef) {
            baseRef.eval = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                // eslint-disable-next-line prefer-spread
                return that.eval.apply(that, args);
            };
        }
        try {
            ret = ret.apply(baseRef, args);
        }
        catch (e) {
            var pos = ast.pos || posUnknown;
            var text = getRefText(ast);
            var err = "on ".concat(text, " at L/N ").concat(pos.first_line, ":").concat(pos.first_column);
            e.message += err;
            throw e;
        }
        return ret;
    };
    References.prototype._throw = function (ast, property, errorName) {
        if (this.config.env !== 'development') {
            return;
        }
        var text = getRefText(ast);
        var pos = ast.pos || posUnknown;
        var propertyName = property.type === 'index' ? property.id.value : property.id;
        var errorMsg = 'get property ' + propertyName + ' of undefined';
        if (errorName === 'TypeError') {
            errorMsg = propertyName + ' is not method';
        }
        errorMsg += '\n  at L/N ' + text + ' ' + pos.first_line + ':' + pos.first_column;
        var e = new Error(errorMsg);
        e.name = errorName || 'ReferenceError';
        throw e;
    };
    return References;
}(Compile));
export { References };
applyMixins(Compile, [References]);
//# sourceMappingURL=references.js.map