import { parse as _parse } from './parse/index';
var blockTypes = {
    if: true,
    foreach: true,
    macro: true,
    noescape: true,
    define: true,
    macro_body: true,
};
var customBlocks = {};
var TRIM_REG = /^[ \t]*\n/;
/**
 * @param str string to parse
 * @param blocks self define blocks, such as `#cms(1) hello #end`
 * @param ignoreSpace if set true, then ignore the newline trim.
 */
export var parse = function (str, blocks, ignoreSpace) {
    if (ignoreSpace === void 0) { ignoreSpace = true; }
    var asts = _parse(str);
    customBlocks = blocks !== null && blocks !== void 0 ? blocks : {};
    /**
     * remove all newline after all direction such as `#set, #each`
     */
    if (ignoreSpace) {
        asts.forEach(function (ast, i) {
            if (typeof ast === 'string') {
                return;
            }
            // after raw and references, then keep the newline.
            if (!['references', 'raw'].includes(ast.type)) {
                var _ast = asts[i + 1];
                if (typeof _ast === 'string' && TRIM_REG.test(_ast)) {
                    asts[i + 1] = _ast.replace(TRIM_REG, '');
                }
            }
        });
    }
    var ret = makeLevel(asts)[0];
    return ret;
};
function makeLevel(block, index) {
    if (index === void 0) { index = 0; }
    var len = block.length;
    var ret = [];
    var ignore = index - 1;
    for (var i = index; i < len; i++) {
        if (i <= ignore)
            continue;
        var ast = block[i];
        var isString = typeof ast === 'string';
        var type = !isString ? ast.type : '';
        var isBlockType = blockTypes[type];
        // support custom block , for example
        // const vm = '#cms(1)<div class="abs-right"> #H(1,"第一个链接") </div> #end'
        // parse(vm, { cms: true });
        if (!isString && type === 'macro_call' && customBlocks[ast.id]) {
            isBlockType = true;
            ast.type = ast.id;
            ast.id = '';
        }
        if (!isBlockType && type !== 'end') {
            ret.push(ast);
            continue;
        }
        if (type === 'end') {
            return [ret, i];
        }
        var _a = makeLevel(block, i + 1), nest = _a[0], current = _a[1];
        ignore = current;
        nest.unshift(block[i]);
        ret.push(nest);
    }
    return [ret, 0];
}
//# sourceMappingURL=parse.js.map